%% 
% HOSWIBRA: branch switching from Hopf points
% 
%  p=hoswibra(dir,fname,ds,para,varargin)
%
%  additional arguments aux include: 
%    z(1,2) for double Hopf: coefficients to mix the two EVecs at bif for the predictor 
%
function p=hoswibra(dir,fname,ds,para,varargin)
if nargin>5; aux=varargin{2}; ndir=varargin{1}; else; aux=0; end 
p=loadp(dir,fname); nu=p.nu;  
if(~isempty(varargin)); [p,ok]=setfn(p,varargin{1}); 
    if ~ok; fprintf('could not change dir-name!\n'); return; end
else fprintf('warning: problem directory unchanged.\n'); end
p=hostanparam(p,aux); % set standard values, also taking aux into account 
nqh=p.hopf.nqh; tl=p.hopf.tl; 
pcheck=1; if isfield(aux,'pcheck'); pcheck=aux.pcheck; end; 
p.hopf.xi=p.hopf.xif/(nu*tl); % seems most reasonable
lam=getlam(p); p=resetc(p); brout=[bradat(p); p.fuha.outfu(p,p.u)]; brout(1,1)=0; 
p.branch=brout; p.file.count=0; p.sol.ptype=3; p.fuha.savefu(p);p.file.count=1; 
r=resi(p,p.u); Gu=getGu(p,p.u,r); [ineg,muv,V]=spcalc(Gu,p,p.sol.j0); 
om=imag(muv(1)); mu0=real(muv(1)); 
phi=V(:,1); phi=phi/norm(phi,2); 
if isfield(aux,'dlam') % user guess for normal form coeffis 
    dlam=aux.dlam; al=10; 
    if isfield(aux,'al'); al=aux.al; fprintf('user set alpha=%g\n',al); end; 
    fprintf('user set dlam=%i, al=%g\n',dlam,al); 
else  % compute normal form coeffis 
  hodel=1e-4; if isfield(aux,'hodel'); hodel=aux.hodel; end 
  [dlam,al]=hogetnf(p,Gu,om,mu0,phi,hodel);
end 
fprintf('real(mu)=%g, imag(mu)=%g\n',mu0,imag(muv(1)));  
fprintf('lam=%g, om=%g,  NF-coeffis: dlam=%i, alpha=%g\n',lam, om, dlam,al); 
if isfield(aux,'nqnew'); p.nc.nq=aux.nqnew; end % reset nq for Mu_t=-resi, 
tl=p.hopf.tl; t=linspace(0,1,tl); 
p.hopf.t=t; p.hopf.T=2*pi/abs(om); p.hopf.y=p.u(1:nu)*ones(1,tl); 

if ~isfield(aux,'z') % standard initial guess
  ig=ds*al*real(phi(1:nu)*exp(-2i*sign(om)*pi*t));  
else % initial guess, version for DOUBLE Hopf
  ig=ds*al*real(phi(1:nu)*(aux.z(1)*exp(-2i*sign(om)*pi*t)+aux.z(2)*exp(2i*sign(om)*pi*t)));  
end 

p.hopf.y=p.hopf.y+ig; p.fuha.ufu=@hostanufu; p.fuha.headfu=@hostanheadfu; 
p.hopf.aux=[]; 
hoplot(p,p.plot.pfig,p.plot.pcmp,p.hopf.aux); title('initial guess'); 
hoplot(p,10,2,p.hopf.aux); title('initial guess'); 
p.sw.para=para; p.sw.spcalc=0; p=hoMini(p); p.sol.ds=ds; par=p.u(p.nu+1:end); 
if para==3;  p.sol.restart=1;   % ---------- nat. param, use TOM 
  p.hopf.y=[p.hopf.y; 
            p.hopf.y(:,1)*ones(1,tl); % append aux vars
            p.hopf.T*ones(1,tl)]; % append T 
  fprintf('nat.parametr, lambda-guess=%g\n',lam+dlam*ds); 
  p.hopf.lam=lam+dlam*ds^2;       
  f=horhs(0,p.hopf.y(:,1),p,par); p.hopf.u0dot=f(1:nu); 
  %p.hopf.tom.Stats='off'; p.hopf.tom.Stats_step='off';
  %p.hopf.tom.Stats='on'; p.hopf.tom.Stats_step='on';
else p.sol.restart=0;           % --------- arcl, use honloopext 
  tau=zeros(1,nu*tl+2+nqh);     % the big tangent vector
  for i=1:tl; si=(i-1)*nu+1; % fill u-part of tangent 
      tau(si:si+nu-1)=ig(:,i)/ds;  end; % honorm(p,tau)
  tau(nu*tl+2)=dlam*ds^2; % the lam part, don't set T yet 
  tau=tau/honorm(p,tau); p.hopf.tau=tau; lamg=lam+ds^2*tau(nu*tl+2); 
  fprintf('arc.parametr, lambda-guess=%g\n',lamg); 
  p.hopf.y=p.u(1:nu)*ones(1,tl)+p.sol.ds*v2tom(p,p.hopf.tau); 
  p=setlam(p,lamg); 
  p.hopf.y0d=sety0dot(p,p.hopf.y,par,p.hopf.T); % phase cond 
  [pc,pc_y]=hopc(p,p.hopf.y,p.hopf.T,lamg); 
  if pcheck % check predictor 
    y0=p.u(1:nu)*ones(1,tl); % vals at HP (only u)
    y=y0+ds*v2tom(p,p.hopf.tau); %max(abs(p.hopf.tau))
    p.hopf.y=y; p.hopf.lam=lam+1*ds*tau(nu*tl+2); 
    hoplot(p,4,p.plot.pcmp,p.hopf.aux); title('initial guess'); 
    f=tomassempbc(p,p.hopf.y,p.hopf.T,p.hopf.lam); 
    normf=norm(f,inf); normy=max(max(abs(y(1:nu,:)-y0(1:nu,:)))); 
    fprintf('max|y-u0|=%g, res=%g\n',normy,normf); 
    p.hopf.y0d=sety0dot(p,p.hopf.y,par,p.hopf.T);
    [pc,pc_y]=hopc(p,p.hopf.y,p.hopf.T,p.hopf.lam); 
    if p.hopf.nqh>0; qf=p.hopf.qfh(p,y); fprintf('q=%g\n',qf); end 
  end
  p.hopf.y=p.u(1:nu)*ones(1,tl); % reset y (only compute tangent for arclength!) 
end