%%
% GETGU: compute jacobian of G, pde-part and auxiliart part separated.
%
%  Gu=getGu(p,u,r)
%
%  p=problem structure
%  u=vector of unknowns
%  r=residual (actually only needed for numjac) 
%
% Important settings (see stanparam): p.sw.spjac, p.nc.nq, p.sw.spcont
%
% See also getGupde, pderesi, stanparam
function Gu=getGu(p,u,r)
global pj phij; 
if (p.sw.spcont==0) % regular continuation
   Gu=getGupde(p,u,r); % get pde part linearization
   if p.nc.nq>0 % derivatives w.r.t active auxiliary vars by finite diff.
     Gqw=zeros(p.nu+p.nc.nq,p.nc.nq); % (nu+nq) x nq
     qu=zeros(p.nc.nq,p.nu); 
     for k=2:p.nc.nq+1 % derivative wrt parameters, skip primary parameter
       % ej(j,n)=jth canonical basis vector of length n
       r1=resi(p,u+p.nc.del*ej(p.nu+p.nc.ilam(k),length(u))); 
       Gqw(:,k-1)=(r1-r)/p.nc.del;
     end
     rq=r(p.nu+1:end); 
     if p.sw.qjac==1; qu=p.fuha.qfder(p,u); % analytical q_u 
     else % numerical q_u 
         for j=1:p.nu 
            r1=p.fuha.qf(p,u+p.nc.del*ej(j,length(u))); 
            qu(:,j)=(r1-rq)/p.nc.del; 
         end
     end
     Gu=[[Gu; qu] Gqw];
   end 
   return;
end % regular continuation done.
if (p.sw.spcont~=0) 
    % fold/branch point-continuation extension of Gu. Implemented explicitly here due to
    % pde/aux structure with analytically known pde-part linearization. The
    % assumption is that initially, this will be called after spcontini, so
    % that the old primary parameter lies at p.nc.ilam(2).
    %
    % Notation: 
    %  v=upde normal unknowns part
    %  w=normal active uaux part
    %  ph=u(p.nu/2+1:p.nu) upde part of kernel eigenvector
    %  om=u(p.nu+p.naux+1:length(u)) active non-primary uaux part of kernel eigenvector 
    %  G=normal pde equations
    %  Q=normal auxiliary equations
    %  Gv=partial derivative G_v, others analogous
    %  Gvvph=G_vv*ph, etc.
    % (linear solve for Newton step of this can be optimized due to block 
    %  diagonal structure as in Keller '86)
    [p,spcont,prim]=spreduce(p); % set regular case sizes and turn off spcont in this block
    u1=[u(1:p.nu);u(2*p.nu+1:2*p.nu+p.naux)]; % non-sp variables (no phi)
    nu1=length(u1); % this is p.nu+p.naux (with p.nu halved)
    r1=[r(1:p.nu);r(2*p.nu+1:2*p.nu+p.nc.nq)]; % normal variables
    ph=u(p.nu+1:2*p.nu); om=u(2*p.nu+p.naux+1:end); 
    Gv=getGupde(p,u1,r1); % get analytic pde part linearization
    %JR    Gvph=r(p.nu+1:2*p.nu); % take reference from residual
    Gvph=Gv*u(p.nu+1:2*p.nu); % take reference from residual
    if p.sw.spjac==1; Gvvph=p.fuha.spjac(p,u); % user def. 
    else % expensive way
     Gv=getGupde(p,u1,r1); % get pde part linearization
     Gvph=Gv*u(p.nu+1:2*p.nu); % take reference from residual
     if 0; % use numjac (not relieable cause Gvvph is not sparse!) 
        thresh=p.nc.del*ones(p.nu,1); 
        phij=ph; pj=p;  M=getM(p); spmat=ones(p.nc.neq);
        np=p.nu/p.nc.neq; M=kron(spmat,M(1:np,1:np));  S=M^2>0; unj=u(1:p.nu); 
        [Gvvph,njfac,njG]=numjac('spnj',0,unj(:),Gvph,thresh,[],0,S,[]); 
     else Gvvph=sparse(p.nu,p.nu); % home-made 
       for j=1:p.nu  
         Gv1=getGupde(p,u1+p.nc.del*ej(j,nu1),r1); % perturbed pde-part lin.
         r2=(Gv1*ph-Gvph)/p.nc.del;
         % add column with finite diff. approx.:
         Gvvph=Gvvph+sparse(1:p.nu,j,r2,p.nu,p.nu); % Gvvph(:,j-1)=(r1-r)/del; 
       end   
     end
    end
    % finite difference for mixed pde-aux part G_vw*ph/om
    Gvwph=sparse(p.nu,p.nc.nq+1); Gwvom=sparse(p.nu,p.nu);   
    % perturb in initial run after spcontini by old primary parameter
    Gv1=getGupde(p,u1+p.nc.del*ej(p.nu+p.nc.ilam(1),nu1),r1); 
    % part of mixed deriv. multiplied with ph (pde part of evec.)
    r1=(Gv1*ph-Gvph)/p.nc.del; 
    Gvwph=Gvwph+sparse(1:p.nu,1,r1,p.nu,p.nc.nq+1);
    if(p.nc.nq==0)
      % Gw finite diff via pderesi:
      r2=pderesi(p,u1+p.nc.del*ej(p.nu+p.nc.ilam(1),nu1));
      Gw=sparse(1:p.nu,1,(r2-r(1:p.nu))/p.nc.del,p.nu,1);      
      % assemble Gu from block matrices; last row is the normalization
      zeroGv=sparse(p.nu,p.nu); zerovt=sparse(1,p.nu); 
      Gu=[[ Gv   zeroGv  Gw  ];
          [Gvvph   Gv   Gvwph];
          [zerovt 2*ph'   0  ]];
      %size(Gu), pause
      p=spextend(p,spcont,prim); % reset p
      return; 
    end
    % now case nq>0; 
    % exclude new primary parameter, and initially also old one:
    for k=1:p.nc.nq 
           Gv1=getGupde(p,u1+p.nc.del*ej(p.nu+p.nc.ilam(k+1),nu1),r1); 
           % part of mixed deriv. multiplied with ph (pde part of evec.)
           r2=(Gv1*ph-Gvph)/p.nc.del;
           Gvwph=Gvwph+sparse(1:p.nu,k+1,r2,p.nu,p.nc.nq+1);
           % part of mixed deriv. multiplied with components of om (aux part of evec.)
           Gwvom=Gwvom+u(2*p.nu+p.naux+k)*(Gv1-Gv)/p.nc.del;
    end
    % Handle case of auxiliary functions
    Qvvph=sparse(p.nc.nq,p.nu); Qwvom=sparse(p.nc.nq,p.nu); Qvwph=sparse(p.nc.nq,p.nc.nq+1);
    rq=r(2*p.nu+1:2*p.nu+p.nc.nq);       % aux. eqn. residual
    Qvph=p.fuha.qfder(p,u1)*ph;          % aux. eqn. linearization residual
 % this is in fact also r(2*p.nu+p.nc.nq+1:length(r)-1)', but an initially poor guess 
 % might cause divergence
     if p.sw.qjac==1; % analytical q_u 
       Qv=p.fuha.qfder(p,u1); spqjac=0; if isfield(p.sw,'spqjac'); spqjac=p.sw.spqjac; end 
       if spqjac==1; Qvvph=p.fuha.spqjac(p,u);  % analytical pa_u(q_u*phi) 
       else
         for j=1:p.nu
           Qv1=p.fuha.qfder(p,u1+p.nc.del*ej(j,nu1)); % perturbed pde-part lin.
           r2=(Qv1*ph-Qvph)/p.nc.del;
           Qvvph=Qvvph+sparse(1:p.nc.nq,j,r2,p.nc.nq,p.nu);
         end
       end
       Qv1=p.fuha.qfder(p,u1+p.nc.del*ej(p.nu+p.nc.ilam(1),nu1)); 
       r2=(Qv1*ph-Qvph)/p.nc.del;
       Qvwph=Qvwph+sparse(1:p.nc.nq,1,r2,p.nc.nq,p.nc.nq+1); % parameter der. of q_u*phi
       for k=1:p.nc.nq
           Qv1=p.fuha.qfder(p,u1+p.nc.del*ej(p.nu+p.nc.ilam(k+1),nu1)); 
           r2=(Qv1*ph-Qvph)/p.nc.del;
           Qvwph=Qvwph+sparse(1:p.nc.nq,k+1,r2,p.nc.nq,p.nc.nq+1);
           Qwvom=Qwvom+u(2*p.nu+p.naux+k)*(Qv1-Qv)/p.nc.del;
       end
    else % all second order numerically
       Qv=sparse(p.nc.nq,p.nu); 
       for j=1:p.nu
           r1=p.fuha.qf(p,u1+p.nc.del*ej(j,nu1));
           Qv=Qv+sparse(1:p.nc.nq,j,(r1-rq)/p.nc.del,p.nc.nq,p.nu);           
           for jj=1:p.nu  % second derivatives: SLOW!
              rpp=p.fuha.qf( p,u1+p.nc.del*(ej(j,nu1)+ej(jj,nu1)) ); 
              rmm=p.fuha.qf( p,u1-p.nc.del*(ej(j,nu1)+ej(jj,nu1)) ); 
              rpm=p.fuha.qf( p,u1+p.nc.del*(ej(j,nu1)-ej(jj,nu1)) );
              rmp=p.fuha.qf( p,u1-p.nc.del*(ej(j,nu1)-ej(jj,nu1)) );
              r2=u(p.nu+j)*(rpp-rmp-rpm+rmm)/(4*p.nc.del^2);
              Qvvph=Qvvph+sparse(1:p.nc.nq,j,r2,p.nc.nq,p.nu);
           end
           %   mixed derivatives:
           %   first with respect to in initial run the old primary parameter
           kk=p.nu+p.nc.ilam(1);
           rpp=p.fuha.qf( p,u1+p.nc.del*(ej(j,nu1)+ej(kk,nu1)) ); 
           rmm=p.fuha.qf( p,u1-p.nc.del*(ej(j,nu1)+ej(kk,nu1)) ); 
           rpm=p.fuha.qf( p,u1+p.nc.del*(ej(j,nu1)-ej(kk,nu1)) );
           rmp=p.fuha.qf( p,u1-p.nc.del*(ej(j,nu1)-ej(kk,nu1)) );
           r2=u(p.nu+j)*(rpp-rmp-rpm+rmm)/(4*p.nc.del^2);
           Qvwph=Qvwph+sparse(1:p.nc.nq,1,r2,p.nc.nq,p.nc.nq+1);
           %   now with respect to other old active aux. vars
           for k=2:p.nc.nq+1 
              kk=p.nu+p.nc.ilam(k);
              rpp=p.fuha.qf( p,u1+p.nc.del*(ej(j,nu1)+ej(kk,nu1)) ); 
              rmm=p.fuha.qf( p,u1-p.nc.del*(ej(j,nu1)+ej(kk,nu1)) ); 
              rpm=p.fuha.qf( p,u1+p.nc.del*(ej(j,nu1)-ej(kk,nu1)) );
              rmp=p.fuha.qf( p,u1-p.nc.del*(ej(j,nu1)-ej(kk,nu1)) );
              r2=u(2*p.nu+p.naux+k-1)*(rpp-rmp-rpm+rmm)/(4*p.nc.del^2);
              Qwvom=Qwvom+sparse(1:p.nc.nq,j,r2,p.nc.nq,p.nu);
              r2=u(p.nu+j)*(rpp-rmp-rpm+rmm)/(4*p.nc.del^2);
              Qvwph=Qvwph+sparse(1:p.nc.nq,k-1,r2,p.nc.nq,p.nc.nq+1);
           end
       end
    end % qfder=1  
    % Notation: 'lam' here means the (initially) old primary parameter
    % In case p.nc.nq=0 above this was named 'Gw' while here Gw stands for 
    % other aux. vars. derivatives.
    Gwlam=sparse(p.nu,p.nc.nq+1); Gwwom=sparse(p.nu,p.nc.nq+1);
    Qwlam=sparse(p.nc.nq,p.nc.nq+1); Qwwom=sparse(p.nc.nq,p.nc.nq+1);
    for k=2:p.nc.nq+2
        k1=p.nu+p.nc.ilam(k-1);
           % Gw finite diff via pde:
           r1=pderesi(p,u1+p.nc.del*ej(k1,nu1));
           Gwlam=Gwlam+sparse(1:p.nu,k-1,(r1-r(1:p.nu))/p.nc.del,p.nu,p.nc.nq+1);           
           % Qw finite diff:
           r1=p.fuha.qf(p,u1+p.nc.del*ej(k1,nu1));
           Qwlam=Qwlam+sparse(1:p.nc.nq,k-1,(r1-rq)/p.nc.del,p.nc.nq,p.nc.nq+1);
           % second derivatives:
           for kk=3:p.nc.nq+2 % start at 2 to skip primary parameter
              kk1=p.nu+p.nc.ilam(kk-1);
              rpp=p.fuha.qf( p,u1+p.nc.del*(ej(k1,nu1)+ej(kk1,nu1)) );
              rmm=p.fuha.qf( p,u1-p.nc.del*(ej(k1,nu1)+ej(kk1,nu1)) );
              rpm=p.fuha.qf( p,u1+p.nc.del*(ej(k1,nu1)-ej(kk1,nu1)) );
              rmp=p.fuha.qf( p,u1-p.nc.del*(ej(k1,nu1)-ej(kk1,nu1)) );
              r2=u(2*p.nu+p.naux+kk-2)*(rpp-rmp-rpm+rmm)/(4*p.nc.del^2);
              Qwwom=Qwwom+sparse(1:p.nc.nq,k-1,r2,p.nc.nq,p.nc.nq+1);
              % now for Gww... 
              rpp=pderesi( p,u1+p.nc.del*(ej(k1,nu1)+ej(kk1,nu1)) ); 
              rmm=pderesi( p,u1-p.nc.del*(ej(k1,nu1)+ej(kk1,nu1)) ); 
              rpm=pderesi( p,u1+p.nc.del*(ej(k1,nu1)-ej(kk1,nu1)) );
              rmp=pderesi( p,u1-p.nc.del*(ej(k1,nu1)-ej(kk1,nu1)) );
              r2=u(2*p.nu+p.naux+kk-2)*(rpp-rmp-rpm+rmm)/(4*p.nc.del^2);
              Gwwom=Gwwom+sparse(1:p.nu,k-1,r2,p.nu,p.nc.nq+1);
           end
       end
       % remove spcont-parameter; initially the old primary parameter:
       Gw=Gwlam(:,2:end); 
       Qw=Qwlam(:,2:end); 
       
       % assemble Gu from block matrices; last row is the normalization
       zeroGv=sparse(p.nu,p.nu); zeroGw=sparse(p.nu,p.nc.nq); 
       zeroQw=sparse(p.nc.nq,p.nc.nq); zeroQv=sparse(p.nc.nq,p.nu);
       zerovt=sparse(1,p.nu);    zerowt=sparse(1,p.nc.nq+1);
%       Gu=[[    Gv      zeroGv    Gwlam     Gw];
%            [Gvvph+Gwvom   Gv   Gvwph+Gwwom    zeroGw  ];
%            [    Qv      zeroQv    Qwlam     Qw];
%            [Qvvph+Qwvom   Qv   Qvwph+Qwwom    zeroQw  ];
%            [  zerovt     2*ph'    zerowt     2*om']];
       Gu=[[    Gv      zeroGv    Gwlam     zeroGw];
            [Gvvph+Gwvom   Gv   Gvwph+Gwwom    Gw  ];
            [    Qv      zeroQv    Qwlam     zeroQw];
            [Qvvph+Qwvom   Qv   Qvwph+Qwwom    Qw  ];
            [  zerovt     2*ph'    zerowt     2*om']];
           
    p=spextend(p,spcont,prim); % reset p
end % Gu for special continuation
